" Vim syntax file
" Language: FreeVR Internal SDK (freevrlib.vim)
" Maintainer: Bill Sherman
" Latest Revision: 2022/02/06
"
" USAGE:
"    * Copy to ~/.vim/syntax
"    * :set filetype=freevrlib
"
" TODO:
"    * make a separate version of this file that's for applications
"        - DONE: probably make this one "freevrlib.vim" or something
"    * Make a syntax file for freevr-rc files!
"    * Consider adding syntax definitions for internal input/output types
"    * make the background more subtle when keywords are in commented section
"    * Is there a way to wildcard 'cCppOut*' and 'cComment*' ?
"    * Add "vr_ansi.h" ANSI codes
"    * DONE: make freevrDefine syntax 'containedin' #if... statements
"    * DONE: Fix DEPRECATED to be dramatically flagged (probably because of above)
"    * DONE: check out the "syn cluster" syntax -- can types be grouped!
"    * DONE: make "delete me" a warning
"    * DONE: Make a separate PfFunction type [though presently the same color]
"    * DONE: Use "syntax match" for filenames to include full name
"

" Don't reload once loaded -- except when debugging!
"if exists("b:current_syntax")
"  finish
"endif
"
" Colors:
" 0 -- grey
" 1 -- red
" 2 -- green
" 3 -- yellow
" 4 -- blue
" 5 -- purple
" 6 -- cyan
" 7 -- white
" 8 -- black

""""""""""""""""""""
" Keep the C-lang formatting
source $VIMRUNTIME/syntax/c.vim

""""""""""""""""""""
syntax case match

""""""""""""""""""""
" Color mapping
hi freevrDefine term=bold ctermbg=225 guibg=LightMagenta ctermfg=9
hi freevrEnum term=bold ctermbg=225 guibg=LightMagenta ctermfg=9
hi freevrStruct term=bold ctermbg=225 guibg=LightMagenta ctermfg=6 guifg=DarkCyan
hi freevrTypedef term=bold ctermbg=225 guibg=LightMagenta ctermfg=6 guifg=DarkCyan
hi freevrFunction term=bold cterm=bold ctermbg=225 guibg=LightMagenta
hi freevrFilename term=bold cterm=bold ctermfg=red ctermbg=225 guibg=LightMagenta
hi freevrPfFilename term=bold cterm=bold ctermfg=red ctermbg=225 guibg=LightMagenta
hi freevrWarning term=bold cterm=bold ctermfg=blue ctermbg=red guibg=LightMagenta

""""""""""""""""""""
" Clustered contains
syntax cluster fvrClustFilename contains=cComment,cCommentL,cString,cIncluded
syntax cluster fvrClustInsideComments contains=cComment,cCommentL,cCppOutIf2,cCppOutSkip
"syntax cluster fvrClustCppCondition contains=cPreCondit,cCppSkip,cCppInSkip,cCppOutSkip,cPreProcGroup,cPreConditMatch
syntax cluster fvrClustCppCondition contains=cPreCondit

"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
" Globals and other Warnings(should be RED!)
syn match freevrWarning /delete me/ containedin=@fvrClustInsideComments
"    From vr_context.h
syn keyword freevrWarning vrContext vrThisProc
syn keyword freevrWarning vrThisProc
"    From vr_system.h
syn keyword freevrWarning DEPRECATED containedin=@fvrClustCppCondition,@fvrClustInsideComments

"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
" #defines
syn keyword freevrDefine VERBOSE COREDUMP_IS_GOOD containedin=@fvrClustCppCondition,@fvrClustInsideComments
"    From vr_math.h
syn keyword freevrDefine VR_X VR_Y VR_Z VR_W VR_AZIM VR_ELEV VR_ROLL VR_QX VR_QY VR_QZ VR_QW PI DTOR RTOD VR_EPSILON containedin=@fvrClustCppCondition
syn keyword freevrDefine VRMAT_MAXDIM VRMAT_ROWCOL VRMAT_COLROW containedin=@fvrClustCppCondition
syn keyword freevrDefine vrVectorSet3d vrVectorSetAd vrVector4Set4d vrVector4SetAd vrQuatLengthSq containedin=@fvrClustCppCondition
"    From vr_debug.h
" TODO: Add all the ANSI codes -- will wait for vr_ansi.h for that
syn keyword freevrDefine ALWAYS_DBGLVL AALWAYS_DBGLVL COMMON_DBGLVL DEFAULT_DBGLVL SELDOM_DBGLVL RARE_DBGLVL ALMOSTNEVER_DBGLVL containedin=@fvrClustCppCondition
syn keyword freevrDefine CONFIG_ERROR_DBGLVL ERROR_DBGLVL SPAWN_DBGLVL containedin=@fvrClustCppCondition
syn keyword freevrDefine CONFIG_WARN_DBGLVL PERFORMER_DBGLVL PERFORMER_DETAIL_DBGLVL GLX_DBGLVL VARIABLE_DBGLVL PARSE_DBGLVL PARSE_DETAIL_DBGLVL SELFCTRL_DBGLVL TRACE_DBGLVL containedin=@fvrClustCppCondition
syn keyword freevrDefine MAG_DBGLVL TRACKD_DBGLVL STATIC_DBGLVL XWIN_DBGLVL PINCHG_DBGLVL ASCFOB_DBGLVL FASTRK_DBGLVL SHMEMD_DBGLVL VRUIDD_DBGLVL VRPN_DBGLVL GENERIC_DBGLVL INPUT_DBGLVL CONFIG_DBGLVL CALLBACK_DBGLVL CALLBACK_DETAIL_DBGLVL BARRIER_DBGLVL containedin=@fvrClustCppCondition
syn keyword freevrDefine SERIAL_DBGLVL MATH_DBGLVL OBJSEARCH_DBGLVL TRACKMEM_DBGLVL TRACKLOCKS_DBGLVL containedin=@fvrClustCppCondition
syn keyword freevrDefine FORCE_DBG containedin=@fvrClustCppCondition
"    From vr_enums.h
syn keyword freevrDefine VR_ALLUSERS containedin=@fvrClustCppCondition
syn keyword freevrDefine VRSTARTUP_OKAY VRSTARTUP_BADSHMEM VRSTARTUP_NOPROCS VRSTARTUP_BADPROC VRSTARTUP_BADWINDOW containedin=@fvrClustCppCondition
syn keyword freevrDefine VRSTARTUP_BADINDEV VRSTARTUP_BADINMAP VRSTARTUP_NOLOCKSEM VRSTARTUP_NOBARRSEM containedin=@fvrClustCppCondition
syn keyword freevrDefine SIMRENDER_ANYTHING SIMRENDER_CAVE_OUTLINE SIMRENDER_CAVE_SURFACE SIMRENDER_LIGHTING containedin=@fvrClustCppCondition
syn keyword freevrDefine SIMRENDER_SENSOR_HEAD SIMRENDER_SENSOR_OTHER SIMRENDER_SENSOR_SOLID containedin=@fvrClustCppCondition
syn keyword freevrDefine SIMRENDER_SENSOR_AXES SIMRENDER_INDEV_AXES containedin=@fvrClustCppCondition
syn keyword freevrDefine SIMRENDER_WINDOW_OUTLINES SIMRENDER_WINDOW_FRUSTUMS1 SIMRENDER_WINDOW_FRUSTUMS2 containedin=@fvrClustCppCondition
"    From vr_debug.h
syn keyword freevrDefine vrTrace vrDbg real_printf real_fprintf containedin=@fvrClustCppCondition
"    From vr_shmem.h
syn keyword freevrDefine USE_SHMEM VRSHMEM_USESTRUCT VRTRACKMEM containedin=@fvrClustCppCondition
syn keyword freevrDefine SHM_NONE SHM_SVR4MMAP SHM_SYSVIPC SEM_SYSVIPC containedin=@fvrClustCppCondition
syn keyword freevrDefine SHM_BSDANONMMAP SHM_PF_ARENA SHM_DUMMY containedin=@fvrClustCppCondition
syn keyword freevrDefine SEM_SYSVIPC SEM_WIN32 SEM_TCP containedin=@fvrClustCppCondition
syn keyword freevrDefine SEM_IRIX SEM_POSIX containedin=@fvrClustCppCondition
"    From vr_context.h
syn keyword freevrDefine FREEVR_VERSION_MAJOR FREEVR_VERSION_MINOR FREEVR_VERSION_SUB FREEVR_VERSION containedin=@fvrClustCppCondition
"    From vr_system.h
syn keyword freevrDefine VRCONFIG_SHMEM_SIZE FREEVRVERSION VRSYSTEM_SPAWNALL containedin=@fvrClustCppCondition
syn keyword freevrDefine VR_TIME_SIM VR_TIME_SIM2 VR_TIME_PAUSE VR_TIME_SLEEP VR_TIME_VP0 containedin=@fvrClustCppCondition

syn keyword freevrDefine GFX_PERFORMER containedin=cPreCondit,cCppSkip,cCppInSkip,cCppOutSkip,cPreProcGroup,cPreConditMatch containedin=@fvrClustCppCondition

"    From vr_config.h
syn keyword freevrDefine VRCONFIG_DEFAULTHOME VRCONFIG_HOME_ENVVAR VRCONFIG_RCFILENAME VRCONFIG_RC_ENVVAR containedin=@fvrClustCppCondition
syn keyword freevrDefine DEFAULT_DEBUG_LEVEL DEFAULT_PROCESSOR_LOCK DEFAULT_VISRENMODE DEFAULT_PROC_LOCKING_CMD DEFAULT_PROC_UNLOCKING_CMD containedin=@fvrClustCppCondition
syn keyword freevrDefine DEFAULT_USER_NAME DEFAULT_PROP_NAME DEFAULT_INDEV_NAME DEFAULT_WINDOW_NAME DEFAULT_EYELIST_NAME DEFAULT_INPUTPROC_NAME DEFAULT_VISRENPROC_NAME DEFAULT_TELNETPROC_NAME containedin=@fvrClustCppCondition
"    From vr_parse.h
syn keyword freevrDefine PARSE_CONCAT_STRINGS containedin=@fvrClustCppCondition
"    From vr_procs.h
syn keyword freevrDefine VRPROC_NUMTRACEMSGS MP_PTHREADS MP_PTHREADS2 containedin=@fvrClustCppCondition
syn keyword freevrDefine MP_NONE containedin=@fvrClustCppCondition
"    From vr_visren.h
syn keyword freevrDefine VR_MAXSTATS VR_MAXPERSPDEPTH containedin=@fvrClustCppCondition
syn keyword freevrDefine USE_FRUSTUMEYE containedin=@fvrClustCppCondition,@fvrClustInsideComments
syn keyword freevrDefine VR_TIME_INIT VR_TIME_FRAME VR_TIME_RENDER1 VR_TIME_RENDER2 VR_TIME_RENDERINFO VR_TIME_WAIT VR_TIME_SYNC VR_TIME_FREEZE VR_TIME_SWAP VR_TIME_TRAVEL containedin=@fvrClustCppCondition
syn keyword freevrDefine VR_SIMMOVE_NOP VR_SIMMOVE_AWAY VR_SIMMOVE_TOWARD VR_SIMMOVE_CENTER VR_SIMMOVE_SETHOME VR_SIMMOVE_GOHOME VR_SIMMOVE_GOHEAD VR_SIMMOVE_POSY VR_SIMMOVE_NEGY VR_SIMMOVE_POSX VR_SIMMOVE_NEGX VR_SIMMOVE_TOGGLEBIT VR_SIMMOVE_TOGGLEMASK VR_SIMMOVE_SETMASK containedin=@fvrClustCppCondition
"    From vr_visren.c
syn keyword freevrDefine DEERING CRUZNEIRA containedin=@fvrClustCppCondition,@fvrClustInsideComments
"    From vr_input.h
syn keyword freevrDefine INPUT_UIDESC_LEN containedin=@fvrClustCppCondition
"    From vr_basicgfx.glx.h
syn keyword freevrDefine FREEVR TEST_APP GFX_ONLY containedin=@fvrClustCppCondition
syn keyword freevrDefine WIN_GLX WIN_WGL GFX_OGL containedin=@fvrClustCppCondition

syn keyword freevrDefine SOCK_W32 containedin=@fvrClustCppCondition

"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
" Enums
"    From vr_math.h
syn keyword freevrDefine VRDIR_UNKNOWN VRDIR_FORE VRDIR_BACK VRDIR_UP VRDIR_DOWN VRDIR_LEFT VRDIR_RIGHT
"    From vr_enums.h
syn keyword freevrEnum VRSTATUS_UNITIALIZED VRSTATUS_INITIALIZING VRSTATUS_RUNNING VRSTATUS_TERMINATING VRSTATUS_TERMINATED
syn keyword freevrEnum VRUI_UNKNOWN VRUI_DEFAULT VRUI_SIMULATOR VRUI_SIM VRUI_STATIONARY VRUI_CAVE VRUI_HBD VRUI_HMD
syn keyword freevrEnum VRUI_HAND VRUI_WALL VRUI_IMMERSADESK VRUI_WORKBENCH VRUI_PIT VRUI_DESKTOP
syn keyword freevrEnum def none brief one_line verbose machine diagram file_format everything normal
syn keyword freevrEnum vrObject_en
syn keyword freevrEnum VROBJECT_NONE VROBJECT_CALLBACK VROBJECT_SYSTEM VROBJECT_PROCESS
syn keyword freevrEnum VROBJECT_WINDOW VROBJECT_EYELIST VROBJECT_USER VROBJECT_PROP VROBJECT_INDEV VROBJECT_INPUT
syn keyword freevrEnum VROBJECT_EOL_OBJECTS VROBJECT_ADDRESS VROBJECT_CONTEXT VROBJECT_CONFIGINFO VROBJECT_INPUTINFO
syn keyword freevrEnum VROBJECT_EYEINFO VROBJECT_LOCK VROBJECT_BARRIER VROBJECT_ENDOFLIST
syn keyword freevrEnum vrVisrenMode_en
syn keyword freevrEnum VRVISREN_NONE_SELECTED VRVISREN_DEFAULT VRVISREN_MONO VRVISREN_LEFT VRVISREN_RIGHT
syn keyword freevrEnum VRVISREN_DUALFB VRVISREN_DUALVP VRVISREN_ANAGLYPHIC VRVISREN_CHECKERBOARD VRVISREN_VIBRATE
syn keyword freevrEnum vrFrameBufferType_en
syn keyword freevrEnum VRFB_DEFAULT VRFB_FULL VRFB_LEFT VRFB_RIGHT VRFB_FULL_LEFTEYE VRFB_FULL_RIGHTEYE VRFB_SPLIT_LEFTEYE VRFB_SPLIT_RIGHTEYE
syn keyword freevrEnum vrAnaglyphType_en
syn keyword freevrEnum VRANAGLYPH_DEFAULT VRANAGLYPH_ALL VRANAGLYPH_RED VRANAGLYPH_GREEN VRANAGLYPH_BLUE
syn keyword freevrEnum vrEyeType_en
syn keyword freevrEnum VREYE_DEFAULT VREYE_LEFT VREYE_RIGHT VREYE_CYCLOPS VREYE_MONO VREYE_VIBRATE
"    From vr_callback.h
syn keyword freevrEnum vrFuncType_en  VRFUNC_HANDLE_USR2
syn keyword freevrEnum VRFUNC_ONE_DISPLAY_INIT VRFUNC_ONE_DISPLAY_EXIT VRFUNC_ONE_DISPLAY_FRAME VRFUNC_ONE_DISPLAY VRFUNC_ONE_DISPLAY_EYE VRFUNC_ONE_DISPLAY_SIM
syn keyword freevrEnum VRFUNC_ALL_DISPLAY_INIT VRFUNC_ALL_DISPLAY_EXIT VRFUNC_ALL_DISPLAY_FRAME VRFUNC_ALL_DISPLAY VRFUNC_ALL_DISPLAY_EYE VRFUNC_ALL_DISPLAY_SIM
syn keyword freevrEnum VRFUNC_DISPLAY_INIT VRFUNC_DISPLAY_EXIT VRFUNC_DISPLAY_FRAME VRFUNC_DISPLAY VRFUNC_DISPLAY_EYE VRFUNC_DISPLAY_SIM
"    From vr_parse.h
syn keyword freevrEnum VRTOKEN_PARSEERROR VRTOKEN_EOS VRTOKEN_EOF VRTOKEN_UNKNOWN VRTOKEN_COMMENT VRTOKEN_NIL VRTOKEN_DEFAULT
syn keyword freevrEnum VRTOKEN_FALSE VRTOKEN_TRUE VRTOKEN_NO VRTOKEN_YES VRTOKEN_OFF VRTOKEN_ON
syn keyword freevrEnum VRTOKEN_STRING VRTOKEN_NUMBER VRTOKEN_VARIABLE
syn keyword freevrEnum VRTOKEN_HASH VRTOKEN_OPENCURL VRTOKEN_CLOSECURL VRTOKEN_COMMA VRTOKEN_SEMICOLON VRTOKEN_QUOTE
syn keyword freevrEnum VRTOKEN_OPENPAREN VRTOKEN_CLOSEPAREN
syn keyword freevrEnum VRTOKEN_BIOP_BEGIN VRTOKEN_PLUS VRTOKEN_TIMES VRTOKEN_DIVIDE VRTOKEN_BIT_AND VRTOKEN_BIT_OR VRTOKEN_LOGICAL_AND VRTOKEN_LOGICAL_OR
syn keyword freevrEnum VRTOKEN_SETEQUALS VRTOKEN_PLUSEQUALS VRTOKEN_TIMESEQUALS
syn keyword freevrEnum VRTOKEN_EQUALITY VRTOKEN_INEQUALITY VRTOKEN_GREATER VRTOKEN_GREATEROREQ VRTOKEN_LESS VRTOKEN_LESSOREQ VRTOKEN_BIOP_END
syn keyword freevrEnum VRTOKEN_READCONFIG VRTOKEN_READCONFIGSTR VRTOKEN_NOP VRTOKEN_EXIT VRTOKEN_ABORT VRTOKEN_EXEC VRTOKEN_ECHO VRTOKEN_SETENV VRTOKEN_UNSETENV VRTOKEN_PRINTOBJ VRTOKEN_PRINTCONFIG VRTOKEN_GDBPAUSE
syn keyword freevrEnum VRTOKEN_IF VRTOKEN_IFEXEC VRTOKEN_ELSE
syn keyword freevrEnum VRTOKEN_SET VRTOKEN_SETDEFAULT VRTOKEN_USESYSTEM VRTOKEN_SYSTEM VRTOKEN_PROCESS VRTOKEN_WINDOW VRTOKEN_EYELIST VRTOKEN_USER VRTOKEN_PROP VRTOKEN_INPUTDEV VRTOKEN_INPUTMAP
syn keyword freevrEnum VRTOKEN_NAME VRTOKEN_TYPE VRTOKEN_DSOFILE VRTOKEN_DSOFUNC VRTOKEN_ARGS VRTOKEN_MALLEABLE VRTOKEN_COPYOBJECT
syn keyword freevrEnum VRTOKEN_DEBUG_LEVEL VRTOKEN_DEBUG_EXACT VRTOKEN_EYES VRTOKEN_EXECSTART VRTOKEN_EXECSTOP VRTOKEN_EXECUPONERROR VRTOKEN_EXITUPONERROR VRTOKEN_NEARCLIP VRTOKEN_FARCLIP
syn keyword freevrEnum VRTOKEN_SYSTEM_CAVECONFIG VRTOKEN_SYSTEM_PROCS VRTOKEN_SYSTEM_INPUTMAP VRTOKEN_SYSTEM_MASTER VRTOKEN_SYSTEM_SLAVES
syn keyword freevrEnum VRTOKEN_SYSTEM_LOCKCPU VRTOKEN_SYSTEM_LOCKCMD VRTOKEN_SYSTEM_UNLOCKCMD VRTOKEN_SYSTEM_PRECONTEXT VRTOKEN_SYSTEM_PRECONFIG VRTOKEN_SYSTEM_PREINPUT VRTOKEN_SYSTEM_POSTCONTEXT VRTOKEN_SYSTEM_POSTCONFIG VRTOKEN_SYSTEM_POSTINPUT VRTOKEN_SYSTEM_VISMODE VRTOKEN_WORLD_TRANSFORM VRTOKEN_WORLD_TRANSLATE VRTOKEN_WORLD_ROTATE
syn keyword freevrEnum VRTOKEN_PROCESS_MAIN VRTOKEN_PROCESS_COMPUTE VRTOKEN_PROCESS_TELNET VRTOKEN_PROCESS_VISREN VRTOKEN_PROCESS_AUDREN VRTOKEN_PROCESS_HAPREN VRTOKEN_PROCESS_TASREN VRTOKEN_PROCESS_VESREN
syn keyword freevrEnum VRTOKEN_PROCESS_MACHINE VRTOKEN_PROCESS_TYPE VRTOKEN_PROCESS_SYNC VRTOKEN_PROCESS_USEC VRTOKEN_PROCESS_COLOR VRTOKEN_PROCESS_STRING VRTOKEN_PROCESS_DBGFILE VRTOKEN_PROCESS_OBJECT VRTOKEN_PROCESS_OBJECTS VRTOKEN_PROCESS_STATSARGS
syn keyword freevrEnum VRTOKEN_WINDOW_MOUNT VRTOKEN_WINDOW_GRAPHICS VRTOKEN_WINDOW_EYE_COORDS VRTOKEN_WINDOW_COORDS VRTOKEN_WINDOW_TRANSFORM VRTOKEN_WINDOW_TRANSLATE VRTOKEN_WINDOW_ROTATE VRTOKEN_WINDOW_VIEWPORT VRTOKEN_WINDOW_VIEWPORT_LEFT VRTOKEN_WINDOW_VIEWPORT_RIGHT VRTOKEN_WINDOW_FVIEWPORT VRTOKEN_WINDOW_FVIEWPORT_LEFT VRTOKEN_WINDOW_FVIEWPORT_RIGHT VRTOKEN_WINDOW_VIEWMASK VRTOKEN_WINDOW_COLORMASK
syn keyword freevrEnum VRTOKEN_WINDOW_UISHOW VRTOKEN_WINDOW_UILOC VRTOKEN_WINDOW_UICOL VRTOKEN_WINDOW_FPSSHOW VRTOKEN_WINDOW_FPSLOC VRTOKEN_WINDOW_FPSCOL VRTOKEN_WINDOW_STATSSHOW VRTOKEN_WINDOW_STATSPROCS VRTOKEN_WINDOW_FRAMESHOW VRTOKEN_WINDOW_SIMMASK
syn keyword freevrEnum VRTOKEN_EYELIST_MONOFB VRTOKEN_EYELIST_LEFTFB VRTOKEN_EYELIST_RIGHTFB VRTOKEN_EYELIST_LEFTVP VRTOKEN_EYELIST_RIGHTVP VRTOKEN_EYELIST_ANAGLFB
syn keyword freevrEnum VRTOKEN_INDEV_FUNCTION VRTOKEN_INDEV_DOF VRTOKEN_INPUT VRTOKEN_CONTROL
syn keyword freevrEnum VRTOKEN_INDEV_REFFRAME_TRANSFORM VRTOKEN_INDEV_REFFRAME_TRANSLATE VRTOKEN_INDEV_REFFRAME_ROTATE
syn keyword freevrEnum VRTOKEN_INDEV_RECEIVER_TRANSFORM VRTOKEN_INDEV_RECEIVER_TRANSLATE VRTOKEN_INDEV_RECEIVER_ROTATE
syn keyword freevrEnum VRTOKEN_INDEV_INITPOS_TRANSFORM VRTOKEN_INDEV_INITPOS_TRANSLATE VRTOKEN_INDEV_INITPOS_ROTATE
syn keyword freevrEnum VRTOKEN_USER_IOD VRTOKEN_USER_COLOR VRTOKEN_USER_HEADSENSOR VRTOKEN_USER_BODYSENSOR
syn keyword freevrEnum VRTOKEN_TRAVEL_TRANSFORM VRTOKEN_TRAVEL_TRANSLATE VRTOKEN_TRAVEL_ROTATE VRTOKEN_USER_REFFRAME_TRANSFORM VRTOKEN_USER_REFFRAME_TRANSLATE VRTOKEN_USER_REFFRAME_ROTATE
syn keyword freevrEnum VRTOKEN_PROP_HEADSENSOR VRTOKEN_PROP_REFFRAME_TRANSFORM VRTOKEN_PROP_REFFRAME_TRANSLATE VRTOKEN_PROP_REFFRAME_ROTATE
syn keyword freevrEnum VRTOKEN_INMAP_SWITCH2 VRTOKEN_INMAP_SWITCHN VRTOKEN_INMAP_VALUATOR VRTOKEN_INMAP_POSITION VRTOKEN_INMAP_POINTER VRTOKEN_INMAP_PLANE VRTOKEN_INMAP_KEYBOARD VRTOKEN_INMAP_TEXT
syn keyword freevrEnum VRTOKEN_ENDOFTOKENLIST
"    From vr_procs.h
syn keyword freevrEnum vrProcessType_en
syn keyword freevrEnum VRPROC_NONE VRPROC_NOCONFIG VRPROC_UNKNOWN VRPROC_MAIN VRPROC_PARENT VRPROC_TELNET VRPROC_COMPUTE VRPROC_INPUT VRPROC_VISREN VRPROC_AUDREN VRPROC_HAPREN VRPROC_OLFREN VRPROC_TASREN VRPROC_VESREN
"    From vr_visren.h
syn keyword freevrEnum vrWindowMountType_en
syn keyword freevrEnum VRWINDOW_FIXED VRWINDOW_HEADMOUNT VRWINDOW_HANDHELD VRWINDOW_SIMULATOR
"    From vr_input.h
syn keyword freevrEnum VRINPUT_UNKNOWN VRINPUT_BINARY VRINPUT_2WAY VRINPUT_NWAY VRINPUT_NARY VRINPUT_VALUATOR VRINPUT_KEYSTROKE VRINPUT_TEXT VRINPUT_6SENSOR VRINPUT_NSENSOR VRINPUT_POSITION VRINPUT_POINTER VRINPUT_PLANE VRINPUT_CONTROL
syn keyword freevrEnum VRINPUT_NOMATCH VRINPUT_MATCH_UNABLE VRINPUT_MATCH_ABLE

"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
" Structs
"    From vr_math.h
"syn keyword freevrStruct 
"    From vr_shmem.h
syn keyword freevrStruct vrShmemInfo_st vrBarrier_st
"    From vr_callback.h
syn keyword freevrStruct vrCallback_st vrCallbackList_st
"    From vr_context.h
syn keyword freevrStruct vrContextInfo_st
"    From vr_system.h
syn keyword freevrStruct vrInheritableInfo_st vrSystemInfo_st
"    From vr_entity.h
syn keyword freevrStruct vrUserInfo_st vrPropInfo_st vrEyeInfo_st vrEyelistInfo_st
"    From vr_objects.h
syn keyword freevrStruct vrObjectInfo_st vrObjectLists_st
"    From vr_config.h
syn keyword freevrStruct vrConfigInfo_st
"    From vr_parse.h
syn keyword freevrStruct vrTokenInfo_st vrFunctionInfo_st
"    From vr_procs.h
syn keyword freevrStruct vrProcessStats_st vrProcessInfo_st
"    From vr_visren.h
syn keyword freevrStruct vrWindowInfo_st
"    From vr_input.h
syn keyword freevrStruct vrGenericInput_st vr6sensor_st vrInput_st vrID_st vrInputInfo_st

"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
" Typedefs
"    From vr_math.h
syn keyword freevrTypedef vrDirection vrMatrix vrMatrixComposition vrFrustum 
syn keyword freevrTypedef vrPoint vrPointf vrVector vrVectorf vrVector4 vrQuat vrEuler
syn keyword freevrTypedef vrVQSTransform 
"    From vr_enums.h
syn keyword freevrTypedef vrSystemStatus vrUIStyle vrPrintStyle vrObject
syn keyword freevrTypedef vrVisrenModeType vrFrameBufferType vrAnaglyphType vrEyeType
"    From vr_debug.h
syn keyword freevrTypedef vrCodeString 
"    From vr_shmem.h
syn keyword freevrTypedef vrTime vrLock vrShmemInfo vrBarrier
"    From vr_callback.h
syn keyword freevrTypedef vrCallback vrCallbackList
"    From vr_context.h
syn keyword freevrTypedef vrContextInfo
"    From vr_system.h
syn keyword freevrTypedef vrInheritableInfo vrSystemInfo
"    From vr_entity.h
syn keyword freevrTypedef vrUserInfo vrPropInfo vrEyeInfo vrEyelistInfo
"    From vr_objects.h
syn keyword freevrTypedef vrObjectInfo vrObjectLists
"    From vr_config.h
syn keyword freevrTypedef vrConfigInfo
"    From vr_config.h
syn keyword freevrTypedef vrToken
"    From vr_parse.h
syn keyword freevrTypedef vrTokenInfo vrFunctionInfo vrParseInfo
"    From vr_procs.h
syn keyword freevrTypedef vrProcessType vrProcessStats vrProcessInfo
"    From vr_visren.h
syn keyword freevrTypedef vrWindowMountType vrWindowInfo
syn keyword freevrTypedef vrViewport vrViewportFract vrViewportMask
syn keyword freevrTypedef vrPerspData vrRenderInfo vrVisrenOptsType
"    From vr_input.h
syn keyword freevrTypedef vrInputType vrInputMatch
syn keyword freevrTypedef vrGenericInput vr2switch vrNswitch vrValuator vr6sensor vrNsensor vrControl vrInputDesc
syn keyword freevrTypedef vrInput vrInputDevice vrInputOptsType vrInputDTI
syn keyword freevrTypedef vrInputFunction vrControlFunc vr6sensorConv vrInputInfo

"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
" Functions
"    From vr_math.h
syn keyword freevrFunction vrMatrixCreate vrMatrixDelete vrMatrixEqual vrMatrixCopy vrMatrixSetIdentity
syn keyword freevrFunction vrMatrixCreateIdentity vrMatrixSetIdentityRotationOnly vrMatrixSetAd 
syn keyword freevrFunction vrMatrixSetXYZAERAf vrMatrixSetXYZAERAd vrMatrixSetXYZAERE vrMatrixSetEulerAzimaxis vrEulerSetFromMatrix vrMatrixSetFromCoords vrMatrixSetTranslation3d vrMatrixSetTranslationAd vrMatrixSetTranslationOnlyAd vrMatrixGetResetTranslationAd vrMatrixCompositionFromMatrix vrMatrixSetRotationId vrMatrixSetRotation4d vrMatrixSetRotationAd vrMatrixSetScale3d vrMatrixProduct vrMatrixPreMult vrMatrixPreTranslate3d vrMatrixPreTranslateAd vrMatrixPreRotateId vrMatrixPreRotateAboutOriginId vrMatrixPreRotateAboutPointId vrMatrixPreRotate4d vrMatrixPreScale3d vrMatrixPostMult vrMatrixPostTranslate3d vrMatrixPostTranslateAd vrMatrixPostRotateId vrMatrixPostRotateAboutOriginId vrMatrixPostRotateAboutPointId vrMatrixPostRotate4d vrMatrixPostScale3d vrMatrixTranspose vrMatrixInvert vrMatrixInvertInPlace vrMatrixIsEuclidean vrMatrixMakeEuclidean vrMatrixInvertEuclidean vrMatrixGetScale vrMatrixSetScale vrMatrixDeterminant vr3x3Determinant vr2x2Determinant
syn keyword freevrFunction vrQuatCreate vrQuatSet4dThetaAxis vrQuatSet4d vrQuatProduct vrQuatProductEquals vrQuatCopy vrQuatDelete
syn keyword freevrFunction vrVectorCopy vrVectorNormalize vrVectorLength vrVectorDotProduct vrVectorCrossProduct vrVectorScale vrVectorSubtract vrVectorAddScaledVectors
syn keyword freevrFunction vrPointCreate vrPointCreate3d vrPointCopy vrPointDelete vrPointAddScaledVector vrCoordsSetIdentity
syn keyword freevrFunction vrMatrixSetRotationFromQuat vrMatrixSetFromQuat vrQuatSetFromMatrix vrEulerSetFromQuatPoint vrPointTransformByQuat vrPointTransformByMatrix vrPointGetTransFromMatrix vrMatrixSetTransFromVector vrVectorTransformByMatrix vrVector4TransformByMatrix vrVectorGetTransFromMatrix
syn keyword freevrFunction vrFprintMatrix vrPrintMatrix vrPrintMatrixT vrFprintMatrixT vrPrintQuat vrPrintPoint vrPrintVector vrPrintVector4 vrPrintEuler vrPrintMatrixComposition vrFprintMatrixCompositionOL
syn keyword freevrFunction vrDegreesToRadians vrRadiansToDegrees
"    From vr_utils.h
syn keyword freevrFunction vrShellCmd vrSleep vrAtoI vrAtoP vrBinaryToString vrProcessString vrPrintableString vrStringCharWidth vrStringCharHeight vrSearchStringList vrSearch256StringList vrEndianSwap4 vrEndianSwap8
"    From vr_socket.h
syn keyword freevrFunction vrSocketCreateListen vrSocketCreateListenUDP vrSocketClose vrSocketAwaitData vrSocketAnswer
syn keyword freevrFunction vrSocketCall vrSocketReadMsg vrSocketReadMsgClean vrSocketSendMsg
"    From vr_serial.h
syn keyword freevrFunction vrSerialOpen vrSerialFlushI vrSerialFlushO vrSerialFlushIO vrSerialDrain vrSerialClose
syn keyword freevrFunction vrSerialAwaitData vrSerialMediateWrite vrSerialWrite vrSerialWriteString
syn keyword freevrFunction vrSerialRead vrSerialReadToChar vrSerialReadToCR vrSerialReadToCR_Alt vrSerialReadNbytes_Block
syn keyword freevrFunction vrSerialBaudIntToEnum vrSerialBaudEnumToInt
"    From vr_debug.h
syn keyword freevrFunction vrPrintf vrFprintf vrErrPrintf vrDbgDo
syn keyword freevrFunction vrDbgPrintfN vrDbgPrintf vrMsgPrintf vrDbgFlush vrNormPrintf
syn keyword freevrFunction vrDbgInfo vrDbgStringValue vrDbgValueString vrFatalError
"    From vr_shmem.h
syn keyword freevrFunction vrShmemInit vrShmemAlloc vrShmemAlloc0 vrShmemRealloc vrShmemFree vrShmemExit vrShmemArena vrShmemUsage vrShmemFreed vrFprintShmemInfo 
syn keyword freevrFunction vrShmemStrDup vrShmemStrCat vrShmemMemDup vrShmemIntArray vrShmemFloatArray vrShmemPtrArray
syn keyword freevrFunction vrShmemGetKey vrShmemIdGetSize vrShmemDetach vrShmemRemoveKey vrShmemRemoveId
syn keyword freevrFunction vrLockCreate vrLockCreateName vrLockCreateTraced
syn keyword freevrFunction vrLockFree vrLockFreeAll vrLockCheck vrLockTrace
syn keyword freevrFunction vrLockReadSet vrLockReadRelease vrLockReadToWrite vrLockWriteSet vrLockWriteRelease
syn keyword freevrFunction vrLockStatus vrFprintLock vrFprintLockList vrCountLockList
syn keyword freevrFunction vrBarrierCreate vrBarrierIncrement vrBarrierDecrement vrBarrierFree vrBarrierCheck
syn keyword freevrFunction vrBarrierSync vrBarrierLastToSync vrBarrierFirstToSync vrBarrierReleaseOnce vrBarrierReleaseAll
syn keyword freevrFunction vrFprintBarrier vrFprintBarrierList vrCountBarrierList
"    From vr_callback.h
syn keyword freevrFunction vrCallbackListInitialize vrCallbackCreate vrCallbackCreateNamed
syn keyword freevrFunction vrCallbackSet vrFunctionSetCallback vrCallbackUpdate vrFprintCallbackList vrFprintCallback
syn keyword freevrFunction vrCallbackInvoke vrCallbackInvokeDynamic vrDoNothing vrDoNothingLoud
"    From vr_system.h
syn keyword freevrFunction vrSystemStatusName vrVisrenmodeTypeName vrUIStyleName vrUIStyleValue
syn keyword freevrFunction vrSettingsClear vrFprintSystemSettings vrSystemClear vrSystemCopy vrFprintSystemInfo vrFprintContext vrContextInitialize
syn keyword freevrFunction vrSystemSetName vrSystemSetAuthors vrSystemSetExtraInfo vrSystemSetStatusDescription vrSystemSimCategory vrStart vrExit vrSystemInitialized vrProcessesInitialized vrFrame
syn keyword freevrFunction vrStartTime vrCurrentWallTime vrSimTimeOf vrCurrentSimTime vrTimeSince
"    From vr_entity.h
syn keyword freevrFunction vrPrintStyleName vrPrintStyleValue
syn keyword freevrFunction vrAnaglyphTypeName vrAnaglyphValue 
syn keyword freevrFunction vrEyeTypeName vrEyeValue vrMakeEye vrFprintEyeInfo
syn keyword freevrFunction vrEyelistInitialize vrEyelistClear vrEyelistCopy vrFprintEyelistInfo 
syn keyword freevrFunction vrUserInitialize vrUsersInitialized vrUserClear vrUserCopy vrFprintUserInfo
syn keyword freevrFunction vrPropClear vrPropCopy vrPropFreeze vrFprintPropInfo
syn keyword freevrFunction vrMatrixGetRWFromUserHead vrMatrixGetVWUserFromRWMatrix vrPointGetVWUserFromRWPoint vrPointGetRWFromVWUserPoint vrPointfGetRWFromVWUserPointf vrVectorUserVWFromRW vrPointGetRWLocationFromMatrix vrPointGetVWFromUserMatrix
syn keyword freevrFunction vrMatrixGetUserTravel vrMatrixGetUserTravelInv vrUserTravelReset vrUserTravelTranslate3d vrUserTravelTranslateAd vrUserTravelTranslateVec vrUserTravelRotateId vrUserTravelScale vrUserTravelTransformMatrix vrUserTravelLockSet vrUserTravelLockRelease vrUserTravelFreeze
syn keyword freevrPfFunction vrPfUserTravelTranslate3d vrPfUserTravelReset vrPfUserTravelTranslate3d vrPfUserTravelTranslateAd vrPfUserTravelTranslateVec vrPfUserTravelRotateId vrPfUserTravelScale vrPfUserTravelTransformMatrix
"    From vr_objects.h
syn keyword freevrFunction vrObjectListsInitialize vrObjectTypeName vrRemoveAllObjects vrFprintObjectInfo
syn keyword freevrFunction vrFprintObjectTypeInfo vrObjectFirst vrObjectNew
syn keyword freevrFunction vrObjectListNew vrObjectsAddToList vrObjectSearch vrObjectListSearch vrObjectArraySearch vrObjectClear vrObjectCopy vrObjectDup vrObjectCopyLinkedList
"    From vr_config.h
syn keyword freevrFunction vrConfigInitialize vrConfigure vrFprintConfig vrEvaluateVariable vrSubstituteVariablesInString vrSetNear vrSetFar
syn keyword freevrFunction vrConfigInitialize vrConfigure vrFprintConfig
syn keyword freevrFunction vrEvaluateVariable vrSubstituteVariablesInString vrSetNear vrSetFar
"    From vr_parse.h
syn keyword freevrFunction vrParseEnumeratorString
syn keyword freevrFunction vrTokenStringRepresentation
syn keyword freevrFunction vrParseNextToken vrParseNextTokenParser vrParseReturnPreviousToken vrParseToEOS
syn keyword freevrFunction vrParseMathExpression vrParseSingleStringExpr vrParseStringListExpr vrParseSingleEnumerator vrParseSingleIntegerExpr vrParseIntegerListExpr vrParseSingleFloatExpr vrParseFloatListExpr vrParseFloatListExprAssign vrParseMatrixFromCoords vrParseMatrixTransform vrParseMatrixTranslate vrParseMatrixRotate
syn keyword freevrFunction vrArgParseString vrArgParseFloat vrArgParseFloatList vrArgParseInteger vrArgParseBool vrArgParseChoiceInteger
syn keyword freevrFunction vrParseInputDescription vrParseInputDTI
"    From vr_procs.h
syn keyword freevrFunction vrProcessTypeName vrProcessTypeValue vrProcessClear vrProcessCopy
syn keyword freevrFunction vrProcessCreateMainProcessInfo vrProcessCalcFrameRate vrFprintProcessInfo
syn keyword freevrFunction vrProcessStart vrProcessSync vrProcessStop vrProcessStopAll
syn keyword freevrFunction vrSetSignalHandler vrSigName
syn keyword freevrFunction vrProcessStatsCreate vrProcessStatsMark vrProcessStatsNextFrame
syn keyword freevrFunction vrTelnetTermProc
syn keyword freevrFunction vrThisProcFunc
"    From vr_visren.h
syn keyword freevrFunction vrVisrenInitProc vrVisrenOneFrame vrVisrenTermProc vrVisrenDeviceInList
syn keyword freevrFunction vrFprintRenderInfo vrFprintPerspData
syn keyword freevrFunction vrWindowName vrWindowType vrVisrenModeName vrVisrenModeValue
syn keyword freevrFunction vrWindowClear vrWindowCopy vrFprintWindowInfo
syn keyword freevrFunction vrRenderCurrentWindow vrRenderCurrentUser vrRenderCurrentEye vrRenderCurrentSimTime vrRenderCurrentFrame vrRenderNullWorld vrRenderDefaultSimulator vrRenderText vrRenderSetWindowContext vrRenderGetWindowContext
syn keyword freevrFunction vrRenderTransform vrRenderTransformWindow vrRenderTransformUserTravel vrRenderTransformUserTravelInv vrRenderTransform6sensor vrRenderTransform6sensorDirect vrRenderSetProjectionTransform vrRenderPushPerspFrom6sensor vrRenderPopPersp vrRenderGetBillboardAngles3d vrRenderGetBillboardAnglesAd vrRenderCategory vrVisrenMainLoop vrVisrenGetInfo
syn keyword freevrFunction vrCalcPerspIntermediaries vrCalcPerspFrustumEye vrCalcPerspMatrix
syn keyword freevrFunction vrSimulatorMove vrSimulatorMoveString
"    From vr_input.h
syn keyword freevrFunction vrCreateInputMap vrInputAddDummyToInputMap vrInputInitialize vrInputsInitialized
syn keyword freevrFunction vrInputCreateDataContainerArrayOfType vrInputCountDataContainers vrInputCreateDataContainers vrInputDeviceInList vrGetFuncFromControlList vrInputCreateSelfControlContainers
syn keyword freevrFunction vrInputSystemPauseToggleCallback vrInputPrintContextStructCallback vrInputPrintConfigStructCallback vrInputPrintInputStructCallback
syn keyword freevrFunction vrInputInitProc vrInputTermProc vrInputOneFrame vrInputMainLoop
syn keyword freevrFunction vrInputCheckIfAllInputDevicesAreOpen vrInputWaitForAllInputDevicesToBeOpen vrInputWaitForAllInputsToBeCreated
syn keyword freevrFunction vrGetDefaultInputDeviceInfo vrGetInputDeviceInfo vrInputFreeze
syn keyword freevrFunction vrAssignGenericInput vrAssign2switchValue vrAssignNswitchValue vrAssignValuatorValue vrAssign6sensorValue vrAssign6sensorValueFromValuators
syn keyword freevrFunction vrAssign6sensorR2Exform vrAssign6sensorActiveValue vrAssign6sensorOobValue vrAssign6sensorErrorValue vrAssignNsensorArray
syn keyword freevrFunction vrInputSet2switchDescription vrGet2switchValue vrGet2switchDelta vrGet2switchValueDirect vrGet2switchDeltaDirect vrGet2switchValueNoLastUpdate
syn keyword freevrFunction vrInputSetNswitchDescription vrGetNswitchValue vrGetNswitchDelta vrGetNswitchValueNoLastUpdate
syn keyword freevrFunction vrInputSetValuatorDescription vrGetValuatorValue vrGetValuatorDelta vrGetValuatorValueNoLastUpdate
syn keyword freevrFunction vrInputSet6sensorDescription vrGet6sensorActiveValue vrGet6sensorOobValue vrGet6sensorErrorValue vrGet6sensorDummyValue vrGet6sensor vrMatrixGet6sensorValues vrMatrixGet6sensorValuesNoLastUpdate vrMatrixGet6sensorValuesDirect vrMatrixGet6sensorRawValuesDirect vrGet6sensorValueF vrGet6sensorValueDirectF vrVectorGetRWFrom6sensorDir vrVectorfGetRWFrom6sensorDir vrVectorGetVWFromUser6sensorDir vrVectorfGetVWFromUser6sensorDir vrPointGetRWFrom6sensor vrPointfGetRWFrom6sensor vrPointGetVWFromUser6sensor vrPointfGetVWFromUser6sensor vrEulerGetRWFrom6sensor vrEulerGetVWFromUser6sensor
syn keyword freevrFunction vrInputSetNsensorDescription vrGetNsensorValue vrGetNsensorArray vrGetNsensorValueDirect vrGetNsensorArrayDirect vrGetNsensorValueNoLastUpdate
syn keyword freevrFunction vrInputMakeChecksum vrInputCheckChecksum vrFprintInput vrSprintInputUI
syn keyword freevrFunction vrInputObjectClear vrInputObjectCopy vrInputTypeName vrInputGetFromMapname vrInputGetFromTypeIndex vrFprintInputObject vrFprintInputValue vrFprint6sensorOptions
syn keyword freevrFunction vrInputDeviceClear vrInputDeviceCopy vrFprintInputDevice
syn keyword freevrPfFunction vrPfPointGetRWFrom6sensor vrPfPointGetVWFromUser6sensor vrPfVectorGetRWFrom6sensorDir vrPfVectorGetVWFromUser6sensorDir vrPfMatrixGet6sensorValues vrPfMatrixGet6sensorValuesDirect vrPfMatrixGet6sensorValuesDirectNoLastUpdate vrPfMatrixGet6sensorRawValuesDirect vrPfVectorfGetRWFrom6sensorDir vrPfVectorfGetVWFromUser6sensorDir vrPfPointfGetRWFrom6sensor vrPfPointfGetVWFromUser6sensor vrPfEulerGetRWFrom6sensor vrPfEulerGetVWFromUser6sensor
"    From vr_basicgfx.glx.h
syn keyword freevrFunction vrShapeGLCube vrShapeGLCubeOpen vrShapeGLCubeOutline vrShapeGLCubePlusOutline vrShapeGLPyramid vrShapeGLPyramidOutline vrShapeGLPyramidPlusOutline vrShapeGLFloor vrGLReportAttributes vrGLRenderDefaultSimulator


"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
" Filenames
"syn keyword freevrFilename freevr containedin=cComment,cString,cIncluded
syn match freevrFilename /freevr\.h/ containedin=@fvrClustFilename
syn match freevrFilename /vr_ansi\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_math\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_utils\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_socket\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_serial\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_debug\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_enums\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_shmem\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_callback\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_context\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_system\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_entity\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_objects\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_config\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_parse\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_procs\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_visren\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_visren\..*\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_input.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_basicgfx\..*\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_output\..*\.[ch]/ containedin=@fvrClustFilename
syn match freevrFilename /vr_input\.opts\.h/ containedin=@fvrClustFilename


"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""

" Matches

