" Vim syntax file
" Language: FreeVR Configuration Format (freevrrc.vim)
" Maintainer: Bill Sherman
" Latest Revision: 2022/02/08
"
" USAGE:
"    * Copy to ~/.vim/syntax
"    * :set filetype=freevrrc
"
" TODO:
"    * make a separate version of this file that's for applications
"        - DONE: probably make this one "freevrlib.vim" or something
"    * Make a syntax file for freevr-rc files!
"    * Consider adding syntax definitions for internal input/output types
"    * make the background more subtle when keywords are in commented section
"    * Is there a way to wildcard 'cCppOut*' and 'cComment*' ?
"    * Add "vr_ansi.h" ANSI codes
"    * DONE: make freevrDefine syntax 'containedin' #if... statements
"    * DONE: Fix DEPRECATED to be dramatically flagged (probably because of above)
"    * DONE: check out the "syn cluster" syntax -- can types be grouped!
"    * DONE: make "delete me" a warning
"    * DONE: Make a separate PfFunction type [though presently the same color]
"    * DONE: Use "syntax match" for filenames to include full name
"

" Don't reload once loaded -- except when debugging!
"if exists("b:current_syntax")
"  finish
"endif
"
" Colors:
" 0 -- grey
" 1 -- red
" 2 -- green
" 3 -- yellow
" 4 -- blue
" 5 -- purple
" 6 -- cyan
" 7 -- white
" 8 -- black

syn keyword	confTodo	contained TODO FIXME XXX
" Avoid matching "text#text", used in /etc/disktab and /etc/gettytab
syn match	confComment	"^#.*" contains=confTodo
syn match	confComment	"\s#.*"ms=s+1 contains=confTodo
syn region	confString	start=+"+ skip=+\\\\\|\\"+ end=+"+ oneline
syn region	confString	start=+'+ skip=+\\\\\|\\'+ end=+'+ oneline

" Define the default highlighting.
" Only used when an item doesn't have highlighting yet
hi def link confComment	Comment
hi def link confTodo	Todo
hi def link confString	String

let b:current_syntax = "conf"

""""""""""""""""""""
syntax case ignore

""""""""""""""""""""
" Color mapping
hi fvrcObjectType term=bold ctermbg=225 guibg=LightMagenta ctermfg=6 guifg=DarkCyan
hi fvrcCommand term=bold cterm=bold ctermbg=225 guibg=LightMagenta
hi fvrcSetting term=bold ctermbg=225 guibg=LightMagenta ctermfg=9
hi fvrcObjectType term=bold ctermbg=225 guibg=LightMagenta ctermfg=9
hi fvrcSystemOption term=bold ctermbg=225 guibg=LightMagenta ctermfg=6 guifg=DarkCyan
hi fvrcProcessOption term=bold cterm=bold ctermfg=red ctermbg=225 guibg=LightMagenta
hi freevrPfFilename term=bold cterm=bold ctermfg=red ctermbg=225 guibg=LightMagenta
hi freevrWarning term=bold cterm=bold ctermfg=blue ctermbg=red guibg=LightMagenta

""""""""""""""""""""
" Clustered contains
syntax cluster fvrClustFilename contains=cComment,cCommentL,cString,cIncluded
syntax cluster fvrClustInsideComments contains=cComment,cCommentL,cCppOutIf2,cCppOutSkip
"syntax cluster fvrClustCppCondition contains=cPreCondit,cCppSkip,cCppInSkip,cCppOutSkip,cPreProcGroup,cPreConditMatch
syntax cluster fvrClustCppCondition contains=cPreCondit

"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
" Warnings (should be RED!)

"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
" Commands
syn keyword fvrcCommand usesystem echo set setenv setDefault readconfig
syn keyword fvrcCommand printobject exit abort

"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
" Settings
syn keyword fvrcSetting debuglevel debugthistoo

"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
" Object types
syn keyword fvrcObjectType system process window input inputdevice user prop

"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
" Default options
syn keyword fvrcDefaultOptions visrenmode near far lockcpu lockcommand unlockcommand
syn keyword fvrcDefaultOptions precontext postcontext exituponerror execuponerror execAtStart execAtStop

"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
" System options
syn keyword fvrcSystemOption malleable procs visrenmode inputmap

"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
" Process options
syn keyword fvrcProcessOption type objects printcolor
" also settings like: debuglevel debugthistoo

